import 'package:dio/dio.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:get/get_state_manager/src/simple/get_controllers.dart';
import '../base_view_state.dart';
import 'package:get/get.dart';




///getx的页面状态基类
class BaseViewStateGetxController extends GetxController{

  final rxViewState = ViewState.busy.obs;

  ViewState get viewState => rxViewState.value;


  ///页面报错信息
  ViewStateError viewStateError;

  void onError(ViewStateError viewStateError) {}


  ///--------------------------------------------设置页面状态--------------------------------------------------------------
  void setIdle() {
    rxViewState.value = ViewState.idle;
  }

  void setBusy() {
    rxViewState.value = ViewState.busy;
  }


  /// e为原始报错信息
  void setError({ViewStateErrorType errorType, e}) {

    if(e is DioError){
      e = HttpError.dioError(e);
    }

    if(isEmpty(errorType)){
      if(e is HttpError){
        if(e.code == HttpError.CONNECT_TIMEOUT || e.code == HttpError.RECEIVE_TIMEOUT || e.code == HttpError.SEND_TIMEOUT){
          errorType = ViewStateErrorType.networkTimeOutError;
        }else{
          errorType = ViewStateErrorType.httpError;
        }
      }else{
        errorType = ViewStateErrorType.defaultError;
      }
    }

    rxViewState.value = ViewState.error;

    viewStateError = ViewStateError(
        errorType,
        error: e
    );

    onError(viewStateError);
  }


}