import 'package:flutter/material.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/http/easy_dio.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:flutter_mylibrary/widget/widget_template.dart';

import '../../flutter_mylibrary.dart';
import '../base_view_state.dart';
import '../request_info.dart';
import 'base_view_state_getx_controller.dart';



///用于下拉刷新，上拉加载的控制类
abstract class GetxRefreshListController<T> extends BaseViewStateGetxController with BaseViewState{


  ///自定义第一页页数
  int firstPage() => startPage;

  ///自定义一页数量
  int pageSize() => pageLimit;

  /// 页面数据
  List<T> list = [];

  /// 当前页码
  int _currentPageNum;

  int get currentPageNum => _currentPageNum;


  int _totalNum;

  int get totalNum => _totalNum??0;

  set totalNum(int value) {
    _totalNum = value;
  }


  bool get hasMore => getRequestInfo().useLoadMore?totalNum > list.length? true : false : false;


  onCompleted(List<T> data) {}


  ///---------------------------------------------------数据获取逻辑区-------------------------------------------------------

  void _getData({bool clearData = false, bool clearDataWhenError = false}) async{
    bool requestState = await _request(clearData: clearData, clearDataWhenError: clearDataWhenError);

    if(requestState){
      setIdle();
    }

    refreshController.finishRefresh(success: requestState);
    refreshController.finishLoad(success: requestState, noMore: !hasMore);
  }


  ///获取数据,返回请求是否成功
  Future<bool> _request({bool clearData = false, bool clearDataWhenError = false}) {

    RequestInfo requestInfo = getRequestInfo();

    getOtherData();

    return EasyDio().request(
      requestInfo?.method,
      requestInfo?.url,
      data: requestInfo?.data,
      publicMethod: requestInfo?.publicMethod,
    ).then((res){
      try{
        List<T> resData = [];

        if (isNotEmpty(res.data)) {
          resData = res.data is List?List<T>.from(res.data.map((e) => requestInfo?.handleRes(e))).toList() : requestInfo?.handleRes(res.data);
        }


        if (clearData) {
          list.clear();
        }

        if (isNotEmpty(resData)) {
          list.addAll(resData);
        }

        onCompleted(resData);

        if (isEmpty(list)) {
          ///空数据
          errorType = ViewStateErrorType.noDataError;
          setError(
            errorType: errorType,
          );
        }
        return true;
      }catch(e){

        onError(e);

        if(clearDataWhenError){
          list.clear();
          errorType = ViewStateErrorType.jsonParsingError;
          setError(
              errorType: errorType,
              e: e
          );
        }
        return false;
      }
    }, onError: (e){
      if(clearDataWhenError){
        list.clear();
        setError(
            errorType: errorType,
            e: e
        );
      }

      return false;
    });
  }


  ///------------------------------------------------------刷新，上拉加载----------------------------------------------------

  void refresh({bool clearData = true, bool clearDataWhenError = false}) {
    _currentPageNum = firstPage();
    _getData(clearData: clearData, clearDataWhenError: true);
  }


  /// 上拉加载更多
  void loadMore() {
    _currentPageNum = ++_currentPageNum;
    _getData();
  }

  ///------------------------------------------------------下拉刷新控件----------------------------------------------------

  ///获取一个公共的刷新控件(EasyRefresh)
  Widget getEasyRefreshWidget({ViewStateModelWidgetListBuilder<List<T>> widgetListBuilder, ViewStateModelWidgetBuilder<List<T>> builder, Widget emptyWidget, Widget busyWidget, bool showErrorWidget = true, bool showEmptyWidget = true, bool showBusyWidget = true}){
    Widget emptyErrorWidget = buildRefreshNoDataWidget(viewStateModel: this, busyWidget: busyWidget, emptyDataWidget: emptyWidget, showErrorWidget: showErrorWidget, showEmptyWidget: showEmptyWidget, showBusyWidget: showBusyWidget);
    return EasyRefresh.custom(
      enableControlFinishRefresh: true,
      enableControlFinishLoad: true,
      taskIndependence: true,
      emptyWidget: emptyErrorWidget,
      slivers: isNotEmpty(widgetListBuilder)? widgetListBuilder(list) : [
        isEmpty(emptyErrorWidget)? isNotEmpty(builder)? builder(list) : SliverToBoxAdapter(child: Container(),) :SliverToBoxAdapter(child: Container(),),
      ],
      onRefresh: isBusy? null : () async{
        refresh();
      },
      onLoad: (isBusy || !hasMore)? null : () async{
        loadMore();
      },
      controller: refreshController,
    );
  }






}